//! Create and use invite codes.
//!
//! Implements the following methods:
//!
//! - [`Sbot::invite_create`]
//! - [`Sbot::invite_use`]

use crate::{error::GolgiError, sbot::Sbot, utils};

impl Sbot {
    /// Call the `invite create` RPC method and return the created invite
    pub async fn invite_create(&mut self, uses: u16) -> Result<String, GolgiError> {
        let mut sbot_connection = self.get_sbot_connection().await?;
        let req_id = sbot_connection.client.invite_create_req_send(uses).await?;

        utils::get_async(
            &mut sbot_connection.rpc_reader,
            req_id,
            utils::string_res_parse,
        )
        .await
    }

    /// Call the `invite use` RPC method and return a reference to the message.
    pub async fn invite_use(&mut self, invite_code: &str) -> Result<String, GolgiError> {
        let mut sbot_connection = self.get_sbot_connection().await?;
        let req_id = sbot_connection
            .client
            .invite_use_req_send(invite_code)
            .await?;

        utils::get_async(
            &mut sbot_connection.rpc_reader,
            req_id,
            utils::string_res_parse,
        )
        .await
    }
}
